@@ -1,6 +1,7 @@
 import os
 import numpy as np
 import datetime
+import csv
 import xlrd
 import xlsxwriter
 from dataclasses import dataclass, asdict
@@ -159,6 +160,21 @@ class LEDETVariables:
     variableToSaveTxt: np.ndarray = np.array([])
     typeVariableToSaveTxt: np.ndarray = np.array([])
     variableToInitialize: np.ndarray = np.array([])
+@dataclass
+class Cable:
+    A_CableInsulated: np.ndarray = np.array([])
+    f_SC: np.ndarray = np.array([])
+    f_ST: np.ndarray = np.array([])
+    SCtype: np.ndarray = np.array([])
+    STtype: np.ndarray = np.array([])
+    Tc0_NbTi: np.ndarray = np.array([])
+    Bc20_NbTi: np.ndarray = np.array([])
+    c1_Ic_NbTi: np.ndarray = np.array([])
+    c2_Ic_NbTi: np.ndarray = np.array([])
+    alpha_NbTi: float = 0.59
+    Jc_Nb3Sn0: np.ndarray = np.array([])
+    Tc0_Nb3Sn: np.ndarray = np.array([])
+    Bc20_Nb3Sn: np.ndarray = np.array([])
 
 class ParametersLEDET:
     '''
@@ -204,7 +220,6 @@ class ParametersLEDET:
         if len(LEDETclass.overwrite_f_internalVoids_inGroup) != 0:
             LEDETclass.BlankRows[1, 3:] = LEDETclass.BlankRows[1, 3:] + 2
 
-
     def readLEDETExcel(self, file):
         ##File must be whole eos string
         workbookVariables = xlrd.open_workbook(file)
@@ -291,7 +306,6 @@ class ParametersLEDET:
         self.fileDefaultParameters = os.path.join('ledet', 'variableNamesDescriptions.xlsx')
         self.loadDefaultParameters(self.fileDefaultParameters)
 
-
     def setParameters(self, variablesInputs, variablesOptions, variablesPlots , variablesVariables):
         for k in variablesInputs.keys():
             self.setAttribute(LEDETInputs, k, variablesInputs[k])
@@ -349,6 +363,182 @@ class ParametersLEDET:
         # Set parameters
         self.setParameters(variablesInputs, variablesOptions, variablesPlots, variablesVariables)
 
+    def localsParser(self, locals):
+        for attribute in locals:
+            if attribute in self.Inputs.__annotations__:
+                group = self.Inputs
+            elif attribute in self.Options.__annotations__:
+                group = self.Options
+            elif attribute in self.Plots.__annotations__:
+                group = self.Plots
+            elif attribute in self.Variables.__annotations__:
+                group = self.Variables
+            else:
+                continue
+            tt = type(self.getAttribute(group, attribute))
+            if tt == np.ndarray:
+                    self.setAttribute(group, attribute, np.array(locals[attribute]))
+            else:
+                print(attribute)
+                self.setAttribute(group, attribute, locals[attribute])
+
+    def cpCu_nist_mat(self, T):
+        density = 8960
+        cpCu_perMass = np.zeros(T.size)
+        T[T < 4] = 4
+        idxT1 = np.where(T < 300)
+        idxT2 = np.where(T >= 300)
+        dc_a = -1.91844
+        dc_b = -0.15973
+        dc_c = 8.61013
+        dc_d = -18.996
+        dc_e = 21.9661
+        dc_f = -12.7328
+        dc_g = 3.54322
+        dc_h = -0.3797
+
+        logT1 = np.log10(T[idxT1])
+        tempVar = \
+        dc_a + dc_b * (logT1)**1 + dc_c * (logT1)**2 + dc_d * (logT1)**3 + \
+        dc_e * (logT1)**4 + dc_f * (logT1)**5 + dc_g * (logT1)** 6 + dc_h * (logT1)**7
+        cpCu_perMass[idxT1] = 10**tempVar
+
+        cpCu_perMass[idxT2]= 361.5 + 0.093 * T[idxT2]
+        cpCu = density * cpCu_perMass
+        return cpCu
+
+    def cpNbTi_cudi_mat(self, T, B):
+        Tc0 = 9.2
+        Bc20 = 14.5
+        alpha = .59
+        B[B>= Bc20] = Bc20-10E-4
+
+        Tc = Tc0 * (1 - B / Bc20)**alpha
+        cpNbTi = np.zeros(T.size)
+
+        idxT1 = np.where(T <= Tc)
+        idxT2 = np.where((T > Tc) & (T <= 20.0))
+        idxT3 = np.where((T > 20) & (T <= 50))
+        idxT4 = np.where((T > 50) & (T <= 175))
+        idxT5 = np.where((T > 175) & (T <= 500))
+        idxT6 = np.where((T > 500) & (T <= 1000))
+        idxT7 = np.where(T > 1000)
+
+        p1 = [0.00000E+00,    4.91000E+01,   0.00000E+00,   6.40000E+01,  0.00000E+00]
+        p2 = [0.00000E+00,   1.62400E+01,   0.00000E+00,  9.28000E+02,   0.00000E+00]
+        p3 = [-2.17700E-01,   1.19838E+01,   5.53710E+02, - 7.84610E+03,  4.13830E+04]
+        p4 = [-4.82000E-03,  2.97600E+00, -7.16300E+02,  8.30220E+04,  -1.53000E+06]
+        p5 = [-6.29000E-05, 9.29600E-02, -5.16600E+01,  1.37060E+04,  1.24000E+06]
+        p6 = [0.00000E+00, 0.00000E+00,  -2.57000E-01,  9.55500E+02,  2.45000E+06]
+        p7 = [0, 0, 0, 0, 3.14850E+06]
+
+        cpNbTi[idxT1] = p1[0] * T[idxT1]**4 + p1[1] * T[idxT1]**3 + p1[2] * T[idxT1]**2 + p1[3] * T[idxT1] + p1[4]
+        cpNbTi[idxT2] = p2[0] * T[idxT2]**4 + p2[1] * T[idxT2]**3 + p2[2] * T[idxT2]**2 + p2[3] * T[idxT2] + p2[4]
+        cpNbTi[idxT3] = p3[0] * T[idxT3]**4 + p3[1] * T[idxT3]**3 + p3[2] * T[idxT3]**2 + p3[3] * T[idxT3] + p3[4]
+        cpNbTi[idxT4] = p4[0] * T[idxT4]**4 + p4[1] * T[idxT4]**3 + p4[2] * T[idxT4]**2 + p4[3] * T[idxT4] + p4[4]
+        cpNbTi[idxT5] = p5[0] * T[idxT5]**4 + p5[1] * T[idxT5]**3 + p5[2] * T[idxT5]**2 + p5[3] * T[idxT5] + p5[4]
+        cpNbTi[idxT6] = p6[0] * T[idxT6]**4 + p6[1] * T[idxT6]**3 + p6[2] * T[idxT6]**2 + p6[3] * T[idxT6] + p6[4]
+        cpNbTi[idxT7] = p7[0] * T[idxT7]**4 + p7[1] * T[idxT7]**3 + p7[2] * T[idxT7]**2 + p7[3] * T[idxT7] + p7[4]
+        return cpNbTi
+
+    def quenchPropagationVelocity(self, I, B, T_bath, cable):
+        L0 = 2.44E-08
+        A_CableBare = cable.A_CableInsulated * (cable.f_SC + cable.f_ST)
+        f_SC_inStrand = cable.f_SC / (cable.f_SC + cable.f_ST)
+        f_ST_inStrand = cable.f_ST / (cable.f_SC + cable.f_ST)
+        I = abs(I)
+        J_op = I / A_CableBare
+        Tc = cable.Tc0_NbTi * (1 - B / cable.Bc20_NbTi) ** cable.alpha_NbTi
+        Tcs= (1 - I / (cable.c1_Ic_NbTi + cable.c2_Ic_NbTi * B)) * Tc
+        Ts = (Tcs + Tc) / 2
+        cp_ST = self.cpCu_nist_mat(Ts)
+        cp_SC = self.cpNbTi_cudi_mat(Ts, B)
+        cp = cp_ST * f_ST_inStrand + cp_SC * f_SC_inStrand
+        vQ = J_op / cp * ((L0 * Ts) / (Ts - T_bath))**0.5
+        idxInfQuenchVelocity=np.where(Tcs <= T_bath)
+        vQ[idxInfQuenchVelocity]=1E6
+        return vQ
+
+    def acquireBField(self, ROXIE_File):
+        reader = csv.reader(open(ROXIE_File), delimiter="\t")
+        Inom = self.Options.Iref
+        reader = csv.reader(open(ROXIE_File))
+        B_Field = np.array([])
+        stack = 0
+        for row in reader:
+            if not row: continue
+            row_s = np.array(row[0].split())
+            if not stack:
+                B_Field = np.array(row_s[1:])
+                stack = 1
+            else:
+                B_Field = np.vstack((B_Field, np.array(row_s)))
+        B_Field = B_Field[1:].astype(float)
+        BX = B_Field[:, 5].transpose()
+        BY = B_Field[:, 6].transpose()
+        f_mag_X_all = BX / Inom
+        f_mag_Y_all = BY / Inom
+        f_mag = (f_mag_X_all**2 + f_mag_Y_all**2) ** 0.5
+        B = f_mag * self.Inputs.I00
+
+        B[B > 10E6]=10E-6
+        return B
+
+    def repeatCable(self, cable):
+        nT = self.Inputs.nT
+        nT = nT.astype(int)
+        newCable = Cable()
+        for attribute in cable.__annotations__:
+            if attribute == 'alpha_NbTi': continue
+            x = np.ndarray([])
+            x = getattr(cable, attribute)
+            x = np.repeat(x, nT)
+            setattr(newCable, attribute, x)
+        return newCable
+
+    def adjust_vQ(self, ROXIE_File):
+        cable = Cable()
+        cable.A_CableInsulated = (self.Inputs.wBare_inGroup+2*self.Inputs.wIns_inGroup) \
+                               * (self.Inputs.hBare_inGroup+2*self.Inputs.hIns_inGroup)
+        B = self.acquireBField(ROXIE_File)
+        if max(self.Inputs.nStrands_inGroup) > 1:
+            strandCount = 0
+            GroupCount = 0
+            Bcopy = np.zeros(int(sum(self.Inputs.nT)))
+            for i in range(int(sum(self.Inputs.nT))):
+                Bcopy[i] = sum(B[int(strandCount):int(strandCount+self.Inputs.nStrands_inGroup[GroupCount])])/self.Inputs.nStrands_inGroup[int(GroupCount)]
+                TurnSum = sum(self.Inputs.nT[0:GroupCount+1])
+                if i>TurnSum: GroupCount = GroupCount + 1
+                strandCount = strandCount + self.Inputs.nStrands_inGroup[GroupCount]
+            B = Bcopy
+            cable.f_SC = self.Inputs.f_SC_strand_inGroup * \
+                         (self.Inputs.nStrands_inGroup* (np.pi/4)*(self.Inputs.ds_inGroup**2)) / cable.A_CableInsulated
+            cable.f_ST = (1 - self.Inputs.f_SC_strand_inGroup) * \
+                         (self.Inputs.nStrands_inGroup* (np.pi/4)*(self.Inputs.ds_inGroup**2)) / cable.A_CableInsulated
+        else:
+            cable.f_SC = self.Inputs.f_SC_strand_inGroup * \
+                         (self.Inputs.wBare_inGroup * self.Inputs.hBare_inGroup) / cable.A_CableInsulated
+            cable.f_ST = (1 - self.Inputs.f_SC_strand_inGroup) * \
+                         (self.Inputs.wBare_inGroup * self.Inputs.hBare_inGroup) / cable.A_CableInsulated
+
+        I = self.Inputs.I00
+        T_bath = self.Inputs.T00
+        cable.SCtype = self.Inputs.SCtype_inGroup
+        cable.STtype = self.Inputs.STtype_inGroup
+        cable.Tc0_NbTi = self.Inputs.Tc0_NbTi_ht_inGroup
+        cable.Bc20_NbTi = self.Inputs.Bc2_NbTi_ht_inGroup
+        cable.c1_Ic_NbTi = self.Inputs.c1_Ic_NbTi_inGroup
+        cable.c2_Ic_NbTi = self.Inputs.c2_Ic_NbTi_inGroup
+        cable.alpha_NbTi = .59
+        cable.Jc_Nb3Sn0 = self.Inputs.Jc_Nb3Sn0_inGroup
+        cable.Tc0_Nb3Sn = self.Inputs.Tc0_Nb3Sn_inGroup
+        cable.Bc20_Nb3Sn = self.Inputs.Bc2_Nb3Sn_inGroup
+        cable = self.repeatCable(cable)
+
+        vQ = self.quenchPropagationVelocity(I, B, T_bath, cable)
+        self.setAttribute(getattr(self, "Inputs"), "vQ_iStartQuench", vQ)
+        return #vQ
+
     def addVariablesInputs(self,
                            T00, l_magnet, I00, M_m,
                            fL_I, fL_L,
